import numpy as np
import xarray as xr
import json
import os

# Get the directory of the current script
script_dir = os.path.dirname(os.path.abspath(__file__))

# Construct the path to config.json relative to the script's location
config_path = os.path.join(script_dir, "config.json")

# read config.json file for parameters
with open(config_path) as f:
    config = json.load(f)


def _noise_filter_radar(ds):
    """Filter radar data by noise level.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 radar dataset.

    Returns
    -------
    xr.Dataset
        Radar data filtered by noise level.

    """
    return ds.where(ds.npw1 > config["noise_threshold"])


def _state_filter_radar(ds):
    """Filter radar data for valid state.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 radar dataset.

    Returns
    -------
    xr.Dataset
        Radar data filtered for valid state.
    """

    return ds.where(ds.grst == config["valid_radar_state"])


def _roll_filter(ds, ds_bahamas):
    """Filter any dataset by plane roll angle.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 dataset.
    ds_bahamas : xr.Dataset
        Level0 BAHAMAS dataset.

    Returns
    -------
    xr.Dataset
        Dataset filtered by plane roll angle.
    """

    ds_bahamas_subsampled = ds_bahamas.sel(
        time=ds.time, method="nearest"
    ).assign_coords(time=ds.time)

    return ds.where(np.abs(ds_bahamas_subsampled.IRS_PHI) < config["roll_threshold"])


def _altitude_filter(ds, ds_bahamas):
    """Filter any dataset by plane altitude.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 dataset.
    ds_bahamas : xr.Dataset
        Level0 BAHAMAS dataset.

    Returns
    -------
    xr.Dataset
        Dataset filtered by plane altitude.
    """

    ds_bahamas_subsampled = ds_bahamas.sel(
        time=ds.time, method="nearest"
    ).assign_coords(time=ds.time)

    return ds.where(ds_bahamas_subsampled.IRS_ALT > config["altitude_threshold"])


def coarsen_radiometer(ds):
    """Coarsen radiometer data to 1 Hz.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 radiometer dataset.

    Returns
    -------
    xr.Dataset
        Radiometer data coarsened to 1 Hz.
    """

    return ds.coarsen(time=4, boundary="pad").mean().drop_duplicates("time")


def filter_radar(ds, ds_bahamas):
    """Filter radar data for noise, valid radar states, and roll angle.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 radar dataset.

    Returns
    -------
    xr.Dataset
        Radar data filtered for noise, state, and roll angle.
    """

    return (
        ds.pipe(_noise_filter_radar)
        .pipe(_state_filter_radar)
        .pipe(_roll_filter, ds_bahamas)
        .dropna(dim="time", how="all")
    )


def filter_radiometer(ds, ds_bahamas):
    """Filter radiometer data for height and roll angle.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 radiometer dataset.

    Returns
    -------
    xr.Dataset
        Radiometer data filtered for height and roll angle.
    """

    return (
        ds.pipe(_altitude_filter, ds_bahamas)
        .pipe(_roll_filter, ds_bahamas)
        .dropna(dim="time", how="all")
    )


def correct_radar_height(ds, ds_bahamas):
    """Correct radar range gates with HALO flight altitude to height above WGS84 ellipsoid.

    Parameters
    ----------
    ds : xr.Dataset
        Level0 radar dataset.
    ds_bahamas : xr.Dataset
        Level0 BAHAMAS dataset.

    Returns
    -------
    xr.Dataset
        Radar data corrected to height above WGS84 ellipsoid.

    """
    z_grid = np.arange(0, ds_bahamas.IRS_ALT.max() + 30, 30)
    ds_bahamas_subsampled = ds_bahamas.sel(
        time=ds.time, method="nearest"
    ).assign_coords(time=ds.time)
    flight_los = (
        ds_bahamas_subsampled.IRS_ALT
        / np.cos(np.radians(ds_bahamas_subsampled.IRS_THE))
        / np.cos(np.radians(ds_bahamas_subsampled.IRS_PHI))
    )

    ds_z_grid = xr.DataArray(
        data=np.tile(z_grid, (len(ds.time), 1)),
        dims=("time", "height"),
        coords={"time": ds.time, "height": z_grid},
    )
    ds_range = xr.DataArray(
        coords={"time": ds.time, "height": z_grid},
        dims=("time", "height"),
        data=ds_z_grid
        / np.cos(np.radians(ds_bahamas_subsampled.IRS_THE))
        / np.cos(np.radians(ds_bahamas_subsampled.IRS_PHI)),
    )

    return ds.sel(range=flight_los - ds_range, method="nearest")



import datetime
import re
import zoneinfo


def parse_datestr(datestr):
    """Parse an ISO 8601 date string that may include a IANA time zone.

    Examples:
        The following call will assume local time
        >>> parse_datestr("2024-08-01 09:00:00")
        datetime.datetime(2024, 8, 1, 9, 0)

        Date and time in UTC
        >>> parse_datestr("2024-08-01 09:00:00Z")
        datetime.datetime(2024, 8, 1, 9, 0, tzinfo=datetime.timezone.utc)

        Here, the IANA time zone for Cape Verde Islands will be added
        >>> parse_datestr("2024-08-01 09:00:00[Atlantic/Cape_Verde]")
        datetime.datetime(2024, 8, 1, 9, 0, tzinfo=zoneinfo.ZoneInfo(key='Atlantic/Cape_Verde'))

    """
    # Parse ISO 8601 string and time zone information
    regex = re.compile(r"^(.*?)(?:\[(.*)\])?$")
    iso, tz = regex.match(datestr).groups()

    # Create datetime object and attach timezone info, if available
    date = datetime.datetime.fromisoformat(iso)
    if tz is not None:
        return date.replace(tzinfo=zoneinfo.ZoneInfo(tz))
    else:
        return date



import numpy as np


def _bahamas_fix_time(ds):
    """Fix time coordinate of BAHAMAS datasets."""
    return ds.rename({"tid": "time", "TIME": "time"}).set_index(time="time")


def bahamas(ds):
    """Post-processing of BAHAMAS datasets."""
    return ds.pipe(
        _bahamas_fix_time,
    )


def _radar_ql_fix_time(ds):
    """Fix time coordinate of RADAR QL datasets."""
    datetime = (
        np.datetime64("1970-01-01", "ns")
        + ds.time.values * np.timedelta64(1, "s")
        + ds.microsec.values * np.timedelta64(1, "us")
    )

    return ds.assign(time=datetime).drop_vars("microsec")


def _radar_ql_add_dBZ(ds):
    """Add reflectivity in dB."""
    return ds.assign(
        dBZg=lambda dx: 10 * np.log10(dx.Zg),
        dBZe=lambda dx: 10 * np.log10(dx.Ze),
    )


def radar_ql(ds):
    """Post-processing of Radar quick look datasets."""
    return ds.pipe(
        _radar_ql_fix_time,
    ).pipe(
        _radar_ql_add_dBZ,
    )


def radiometer(ds):
    """Post-processing of radiometer datasets."""
    return ds.rename(number_frequencies="frequency").set_index(frequency="frequencies")
